/* bits.c, bit-level input */

#include "bits.h"
#include <fstream.h>
#include <sys/stat.h>
#include <string.h>

#define BUFFER_SIZE 32768

/* private data */
static unsigned char bfr[BUFFER_SIZE];
static int byteidx;
static int bitidx;
static int bufcount;
static double totbits;
FILE *bitfile;
bool eobs;

static unsigned char writeBuffer[BUFFER_SIZE];
static unsigned int firstRead;
static double totalBytes = 0;
static double numHunks = 0;
static int bugCounter = 0;

static FILE *outfp;

/* return total number of generated bits */
double bitcount()
{
  return totbits;
}

void xfile() {
	outfp=fopen(FileName(), "wb");
}

void WriteOut() {
	if (firstRead == 1) {
	unsigned int x = 0;
	double thisHunk = 0;
	while (x <= sizeof(bfr)) {
		writeBuffer[x]=bfr[x];
		x++;
	}
	
	totalBytes=numHunks * BUFFER_SIZE;
	numHunks++;
	thisHunk=(totbits/8) - totalBytes;
	if (outfp == NULL) { xfile(); }
	fwrite(writeBuffer, thisHunk, 1, outfp);
	}
    firstRead = 1;
}

void FinishWrite() {
	WriteOut();
}


static bool refill_buffer()
{
  WriteOut();
  int i;
  i = fread(bfr, sizeof(unsigned char), BUFFER_SIZE, bitfile);
  if (i <= 0)
  {
    eobs = true;
	FinishWrite();
    return false;
  }
  bufcount = i;
  return true;
}

/* open the device to read the bit stream from it */
void init_getbits(char *bs_filename)
{ 
  if ((bitfile = fopen(bs_filename, "rb")) == NULL)
  {
    printf("Unable to open file %s for reading.\n", bs_filename);
    exit(1);
  }
  byteidx = 0;
  bitidx = 8;
  totbits = 0.0;
  bufcount = 0;
  eobs = false;
  if (!refill_buffer())
  {
    if (eobs)
    {
      printf("Unable to read from file %s.\n", bs_filename);
      exit(1);
    }
  }
}

/*close the device containing the bit stream after a read process*/
void finish_getbits()
{
  FinishWrite();
  if (bitfile)
    fclose(bitfile);
  bitfile = NULL;
  fclose(outfp);
}


int masks[8]={0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80};
int reverseMasks[8] = {1, 2, 4, 8, 16, 32, 64, 128};
int reverseOrder[8] = {7, 6, 5, 4, 3, 2, 1, 0};

unsigned int put1bit(int newVal) {
	unsigned int oldByte = bfr[byteidx];
	unsigned int oldBits[8];
	unsigned int newByte = 0;
	char showMod = ' ';
	int bitPosition = bitidx - 1;
	int oldBit = (bfr[byteidx] & masks[bitidx - 1]) >> (bitidx - 1);

	for (int b=0; b<8; b++) {
		oldBits[b] = (oldByte & masks[b]) >> (b);
		if (b==bitPosition) {
			oldBits[b] = newVal;
			showMod = '*';
		}
		showMod = ' ';
	}
	
    for (int bb=0; bb<8; bb++) {
		newByte = newByte + oldBits[bb] * reverseMasks[bb];
	}
    bfr[byteidx]=newByte;
	int argh = bfr[byteidx];
	bugCounter++;
    return newByte;
}

void setFrameRate (double fr) {
	put1bit(0);
	bitidx--;
	if (fr==29.97) {
		put1bit(1);
		bitidx--;
	}
	else {
		put1bit(0);
		bitidx--;
	}
	if (fr==25) {
		put1bit(1);
		bitidx--;
	}
	else {
		put1bit(0);
		bitidx--;
	}
	if ((fr==25) || (fr==23.976)) {
		put1bit(1);
	}
	else {
		put1bit(0);
	}
	bitidx=bitidx+3;
}


void setAspect (int aspect) {
	put1bit(0);
	bitidx--;
	if (aspect==4) {
		put1bit(1);
		bitidx--;
	}
	else {
		put1bit(0);
		bitidx--;
	}
	if ((aspect==2) || (aspect==3)) {
		put1bit(1);
		bitidx--;
	}
	else {
		put1bit(0);
		bitidx--;
	}
	if ((aspect==1) || (aspect==3)) {
		put1bit(1);
	}
	else {
		put1bit(0);
	}
	bitidx=bitidx+3;
}

void setTC (int tc_val) {
	int k = 0;
	int x = 16777216;
	int j = tc_val;
	int tdt = tc_val; // debug
	int y[25];
	int z = 0;
	int w[25];
	int k_size = 0;
	int final_counter = 0;
	while (k < 25) {
		if (tc_val<x) { w[k]=0; x=x/2; k_size++; }
		else
		{ 	
			if (j%2!=0)
			{
				y[z]=1;
			}
			else
			{
				y[z]=0;
			}
			j=j/2;
			z++;
		}
	    k++;
	}
	z--;
	j=tdt; // debug
	while (z>-1) {
		w[k_size]=y[z];
		z--;
		k_size++;
	}
	while (final_counter<25) {
		if (w[final_counter] == 0) { put1bit(0); }
		if (w[final_counter] == 1) { put1bit(1); }
		get1bit();
		final_counter++;
	}
}

/*read 1 bit from the bit stream */
unsigned int get1bit()
{
  unsigned int bit;

  if (eobs)
    return 0;

  bit = (bfr[byteidx] & masks[bitidx - 1]) >> (bitidx - 1);
  totbits++;
  bitidx--;
  if (!bitidx)
  {
    bitidx = 8;
    byteidx++;
    if (byteidx == bufcount)
    {
      if (bufcount == BUFFER_SIZE)
        refill_buffer();
      else
        eobs = true;
      byteidx = 0;
    }
  }
  return bit;
}

/*read N bit from the bit stream */
unsigned int getbits(int N)
{
 unsigned int val = 0;
 int i = N;
 unsigned int j;

 while (i > 0)
 {
   j = get1bit();
   val = (val << 1) | j;
   i--;
 }
 return val;
}

/*return the status of the bit stream*/
/* returns 1 if end of bit stream was reached */
/* returns 0 if end of bit stream was not reached */

int end_bs()
{
  return eobs;
}

/*this function seeks for a byte aligned sync word (max 32 bits) in the bit stream and
  places the bit stream pointer right after the sync.
  This function returns 1 if the sync was found otherwise it returns 0  */

int seek_sync(unsigned int sync, int N)
{
  unsigned int val;
  unsigned int maxi = (int)pow(2.0, (double)N) - 1;

  while (bitidx != 8)
    get1bit();

  val = getbits(N);

  while ((val & maxi) != sync)
  {
    if (eobs)
      return 0;
    val <<= 8;
    val |= getbits(8);
  }
  return 1;
}

/*look ahead for the next N bits from the bit stream */
unsigned int look_ahead(int N)
{
  unsigned long val = 0;
  unsigned char *tmp_bfr = bfr;
  unsigned char tmp_bfr1[4];
  int j = N;
  int eo_bs = eobs;
  int buf_count = bufcount;
  int bit_idx = bitidx;
  int byte_idx = byteidx;
  fpos_t pos;

  while (j > 0)
  {
    if (eo_bs)
      return 0;
    val <<= 1;
    val |= (tmp_bfr[byte_idx] & masks[bit_idx - 1]) >> (bit_idx - 1);
    bit_idx--;
    j--;
    if (!bit_idx)
    {
      bit_idx = 8;
      byte_idx++;
      if (byte_idx == buf_count)
      {
        if (buf_count == BUFFER_SIZE)
        {
          if (fgetpos(bitfile, &pos))
          {
            printf("Unable to get file position.\n");
            exit(1);
          }
          if (fread(tmp_bfr1, sizeof(unsigned char), 4, bitfile) != 4)
            eo_bs = true;
          else
            tmp_bfr = &tmp_bfr1[0];
          if (fsetpos(bitfile, &pos))
          {
            printf("Unable to set file position.\n");
            exit(1);
          }
        }
        else
          eo_bs = true;
        byte_idx = 0;
      }
    }
  }
  return val;
}
