/******************************************************************************
  PULLDOWN by Hard Code.  Original sources by Brent Beyeler, beyeler@home.com
******************************************************************************/

#include "bits.h"
#include <string.h>
#include <fstream.h>
#include <time.h>

#define FULL_PICTURE_START_CODE   0x00000100
#define FULL_USER_DATA_START_CODE 0x000001B2
#define FULL_SEQUENCE_HEADER_CODE 0x000001B3
#define FULL_SEQUENCE_ERROR_CODE  0x000001B4
#define FULL_EXTENSION_START_CODE 0x000001B5
#define FULL_SEQUENCE_END_CODE    0x000001B7
#define FULL_GROUP_START_CODE     0x000001B8

#define START_CODE_PREFIX    0x000001
#define PICTURE_START_CODE   0x00
#define USER_DATA_START_CODE 0xB2
#define SEQUENCE_HEADER_CODE 0xB3
#define SEQUENCE_ERROR_CODE  0xB4
#define EXTENSION_START_CODE 0xB5
#define SEQUENCE_END_CODE    0xB7
#define GROUP_START_CODE     0xB8

#define SEQUENCE_EXTENSION_ID                   1
#define SEQUENCE_DISPLAY_EXTENSION_ID           2
#define QUANT_MATRIX_EXTENSION_ID               3
#define SEQUENCE_SCALABLE_EXTENSION_ID          5
#define PICTURE_DISPLAY_EXTENSION_ID            7
#define PICTURE_CODING_EXTENSION_ID             8
#define PICTURE_SPATIAL_SCALABLE_EXTENSION_ID   9
#define PICTURE_TEMPORAL_SCALABLE_EXTENSION_ID 10

#define DATA_PARTITIONING    0
#define SPATIAL_SCALABILITY  1
#define SNR_SCALABILITY		  2
#define TEMPORAL_SCALABILITY 3

#define PULLDOWN_LENGTH 4


int verbose_level = 2;
int col, mpeg2;
char oldLevel3[80];
int progressive_sequence, picture_structure, repeat_first_field;
int sequence_headers, sequence_extensions, user_data_bytes;
int vertical_size, horizontal_size, sequence_scalable_extension_present;
int scalable_mode, mb_row, mb_width, previous_macroblock_address;
int mb_column, macroblock_address;
int slice_vertical_position, slice_vertical_position_extension;

double completedFrames = 0;
int temporalDisplay = 0;
double currentFrame = 0;

double framesSoFar = 0;
int GOPSSoFar = 0;
int SequencesSoFar = 0;
char *NewFileName = "pulldown.m2v";
unsigned int time_start;
unsigned int time_end;
double framerate = 29.97;
bool norff = false;
int tff = NULL;
bool telecine = true;
int timecode = 1;
int prog_seq = 0;
int prog_frames = NULL;
double avg_bitrate =0;
double peak_bitrate =0;
double max_bitrate =0;
double gop_bytes = 0;
bool drop_frame = false;
int aspect_ratio=0;

char *FileName()
{
	return NewFileName;
}

double CurrentFrame (double temporalDisplay, double completedFrames) {
  if (completedFrames == 0 && temporalDisplay == 0) 
    currentFrame = 0;
  
//  if (completedFrames != 0 && temporalDisplay == 0) 
//    currentFrame = completedFrames + temporalDisplay+1;

  if (completedFrames != 0 && temporalDisplay == 0) 
  currentFrame = completedFrames + temporalDisplay;


  if (completedFrames == 0 && temporalDisplay != 0) 
    currentFrame = temporalDisplay;
  
//  if (completedFrames != 0 && temporalDisplay != 0) 
//    currentFrame = temporalDisplay + completedFrames + 1;

  if (completedFrames != 0 && temporalDisplay != 0) 
  currentFrame = temporalDisplay + completedFrames;

  return (currentFrame);
}

int InsertPulldown (double currentFrame) {
	
  long x = 0;
  long y = 0;
  long z = PULLDOWN_LENGTH;
  x=currentFrame/z;
  y=(currentFrame - (x * z))+1;
  return(y);  

}

int GetTFF (int pulldownPattern) {
	if (tff) { 
		if (tff==1) { return 1; }
		else { return 0; }
	}
	switch(pulldownPattern) {
       case 1: return 1;
         break;
       case 2: return 1;
         break;
       case 3: return 0;
         break;
	   case 4: return 0;
       }
	 return 0;
}

int GetRFF (int pulldownPattern) {
	if (norff==true) { return 0; }
	switch(pulldownPattern) {
       case 1: return 0;
         break;
       case 2: return 1;
         break;
       case 3: return 0;
         break;
	   case 4: return 1;
       }
	 return 0;
}

double new_timecode (int ThisFrame, double fr, bool drop_frame) {
	int fps, pict, sec, minute, hour, tc; 
	/* convert 23.976/24fps frame number to 29.97/30fps frame number */ 
	if (fr==29.97) { fps = 30; }
	if (fr==23.976) { fps = 24; }
	if (fr==25) { fps = 25; }
    //fps = 30; 
    ThisFrame = (((ThisFrame / 2) * 2 + (ThisFrame / 2) * 3) >> 1); 

    pict = ThisFrame%fps;
    ThisFrame = (ThisFrame-pict)/fps; 
    sec = ThisFrame%60; 
    ThisFrame = (ThisFrame-sec)/60; 
    minute = ThisFrame%60; 
    ThisFrame = (ThisFrame-minute)/60; 
    hour = ThisFrame%24; 
    tc = (drop_frame<<24 | hour<<19) | (minute<<13) | (1<<12) | (sec<<6) | pict; 
	return tc;
}

void zprintf(int level, char *tmpStr, int value)
{
 // if (level <= verbose_level)
    //printf(tmpStr, value);
	if (level == 1)
	{
		printf(tmpStr, value);
	}
}

unsigned int nextbits(int i)
{
  return look_ahead(i);
}


void next_start_code()
{
  if (seek_sync(START_CODE_PREFIX, 24))
    return;
  else
    exit(1);
}

void sequence_header()
{
  int i;

  sequence_headers++;
  zprintf(2, "\n\nsequence_header_code = 0x000001%02X\n", getbits(8));
  horizontal_size = getbits(12);
  mb_width = horizontal_size / 16;
  if (mb_width % 16)
    mb_width++;
  zprintf(2, "  horizontal_size_value = %d\n", horizontal_size);
  zprintf(2, "  SEQUENCES SO FAR = %d\n", SequencesSoFar);
  SequencesSoFar++;
  vertical_size = getbits(12);
  zprintf(2, "  vertical_size_value = %d\n", vertical_size);
  if (aspect_ratio>0) {   setAspect(aspect_ratio); }
  zprintf(2, "  aspect_ratio_information = %d\n", getbits(4));
  setFrameRate(framerate);
  zprintf(2, "  frame_rate_code = %d\n", getbits(4));
  max_bitrate = getbits(18);
  zprintf(2, "  bit_rate_value = %d\n", max_bitrate);
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  vbv_buffer_size_value = %d\n", getbits(10));
  zprintf(2, "  constrained_parameters_flag = %d\n", get1bit());
  i = get1bit();
  zprintf(2, "  load_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d intra_quantiser_matrix bytes = %d\n", i);
  }
  i = get1bit();
  zprintf(2, "  load_non_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d non_intra_quantiser_matrix bytes = %d\n", i);
  }
  next_start_code();
}


void sequence_extension()
{
  sequence_extensions++;
  zprintf(2, "\nextension_start_code = 0x000001%02X\n", getbits(8));
  zprintf(2, "  sequence_extension_id = %d\n", getbits(4));
  zprintf(2, "  profile_and_level_indication = %d\n", getbits(8));
  if (prog_seq==1) {put1bit(1); }
  if ((prog_seq==2) || (prog_seq==0)) { put1bit(0); }
  progressive_sequence = get1bit();
  zprintf(2, "  progressive_sequence = %d\n", progressive_sequence);
  zprintf(2, "  chroma_format = %d\n", getbits(2));
  zprintf(2, "  horizontal_size_extension = %d\n", getbits(2));
  zprintf(2, "  vertical_size_extension = %d\n", getbits(2));
  zprintf(2, "  bit_rate_extension = %d\n", getbits(12));
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  vbv_buffer_size_extension = %d\n", getbits(8));
  zprintf(2, "  low_delay = %d\n", get1bit());
  zprintf(2, "  frame_rate_extension_n = %d\n", getbits(2));
  zprintf(2, "  frame_rate_extension_d = %d\n", getbits(5));
  next_start_code();
}

void sequence_display_extension()
{
  int i;

  zprintf(3, "  sequence_display_extension_id = %01X\n", getbits(4));
  zprintf(2, "  video_format = %d\n", getbits(3));
  i = get1bit();
  zprintf(2, "  colour_description = %d\n", i);
  if (i)
  {
    zprintf(2, "  colour_primaries = %d\n", getbits(8));
    zprintf(2, "  transfer_characteristics = %d\n", getbits(8));
    zprintf(2, "  matrix_coefficients = %d\n", getbits(8));
  }
  zprintf(2, "  display_horizontal_size = %d\n", getbits(14));
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  display_vertical_size = %d\n", getbits(14));
  next_start_code();
}

void sequence_scalable_extension()
{
  int i;

  sequence_scalable_extension_present = 1;
  zprintf(2, "  sequence_scalable_extension_id = %01X\n", getbits(4));
  scalable_mode = getbits(2);
  zprintf(2, "  scalable_mode = %d\n", scalable_mode);
  zprintf(2, "  layer_id = %d\n", getbits(4));
  if (scalable_mode == SPATIAL_SCALABILITY)
  {
    zprintf(2, "  lower_layer_prediction_horizontal_size = %d\n", getbits(14));
    zprintf(2, "  marker bit = %d\n", get1bit());
    zprintf(2, "  lower_layer_prediction_vertical_size = %d\n", getbits(14));
    zprintf(2, "  horizontal_subsampling_factor_m = %d\n", getbits(5));
    zprintf(2, "  horizontal_subsampling_factor_n = %d\n", getbits(5));
    zprintf(2, "  vertical_subsampling_factor_m = %d\n", getbits(5));
    zprintf(2, "  vertical_subsampling_factor_n = %d\n", getbits(5));
  }
  if (scalable_mode == TEMPORAL_SCALABILITY)
  {
    i = get1bit();
    zprintf(2, "  picture_mux_enable = %d\n", i);
    if (i)
      zprintf(2, "  mux_to_progressive_sequence = %d\n", get1bit());
    zprintf(2, "  picture_mux_order = %d\n", getbits(3));
    zprintf(2, "  picture_mux_factor = %d\n", getbits(3));
  }
  next_start_code();
}

void quant_matrix_extension()
{
  int i;

  zprintf(2, "  quant_matrix_extension_id = %01X\n", getbits(4));
  i = get1bit();
  zprintf(2, "  load_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d intra_quantiser_matrix bytes = %d\n", i);
  }
  i = get1bit();
  zprintf(2, "  load_non_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d non_intra_quantiser_matrix bytes = %d\n", i);
  }
  i = get1bit();
  zprintf(2, "  load_chroma_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d chroma_intra_quantiser_matrix bytes = %d\n", i);
  }
  i = get1bit();
  zprintf(2, "  load_chroma_non_intra_quantiser_matrix = %d\n", i);
  if (i)
  {
    for (i = 0; i < 64; i++)
      getbits(8);
    zprintf(2, "    read %d chroma_non_intra_quantiser_matrix bytes = %d\n", i);
  }
  next_start_code();
}

void picture_display_extension()
{
  int i, number_of_frame_centre_offsets;

  zprintf(2, "  picture_display_extension_id = %01X\n", getbits(4));
  if ((progressive_sequence == 1) || (picture_structure != 3))
	 number_of_frame_centre_offsets = 1;
  else
	 if (repeat_first_field == 1)
		number_of_frame_centre_offsets = 3;
	 else
		number_of_frame_centre_offsets = 2;
  for (i = 0; i < number_of_frame_centre_offsets; i++)
  {
    zprintf(2, "  frame_centre_horisontal_offset = %d\n", getbits(16));
    zprintf(2, "  marker bit = %d\n", get1bit());
    zprintf(2, "  frame_centre_vertical_offset = %d\n", getbits(16));
    zprintf(2, "  marker bit = %d\n", get1bit());
  }
  next_start_code();
}

void picture_spatial_scalable_extension()
{
  zprintf(2, "  picture_spatial_scalable_extension_id = %01X\n", getbits(4));
  zprintf(2, "  lower_layer_temporal_reference = %d\n", getbits(10));
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  lower_layer_horizontal_offset = %d\n", getbits(15));
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  lower_layer_vertical_offset = %d\n", getbits(15));
  zprintf(2, "  spatial_temporal_weight_code_table_index = %d\n", getbits(2));
  zprintf(2, "  lower_layer_progressive_frame = %d\n", get1bit());
  zprintf(2, "  lower_layer_deinterlaced_field_select = %d\n", get1bit());
  next_start_code();
}

void picture_temporal_scalable_extension()
{
  zprintf(2, "  picture_temporal_scalable_extension_id = %01X\n", getbits(4));
  zprintf(2, "  reference_select_code = %d\n", getbits(2));
  zprintf(2, "  forward_temporal_reference = %d\n", getbits(10));
  zprintf(2, "  marker bit = %d\n", get1bit());
  zprintf(2, "  backward_temporal_reference = %d\n", getbits(10));
  next_start_code();
}


void extension_data(int i)
{
  while (nextbits(8) == EXTENSION_START_CODE)
  {
    zprintf(2, "\nextension_start_code = 0x000001%02X\n", getbits(8));
    if (i == 0)
    {
      if (nextbits(4) == SEQUENCE_DISPLAY_EXTENSION_ID)
        sequence_display_extension();
      if (nextbits(4) == SEQUENCE_SCALABLE_EXTENSION_ID)
        sequence_scalable_extension();
    }
    if (i == 2)
    {
      if (nextbits(4) == QUANT_MATRIX_EXTENSION_ID)
        quant_matrix_extension();
      if (nextbits(4) == PICTURE_DISPLAY_EXTENSION_ID)
        picture_display_extension();
      if (nextbits(4) == PICTURE_SPATIAL_SCALABLE_EXTENSION_ID)
        picture_spatial_scalable_extension();
      if (nextbits(4) == PICTURE_TEMPORAL_SCALABLE_EXTENSION_ID)
        picture_temporal_scalable_extension();
    }
  }
}

void user_data()
{
  zprintf(2, "\nuser_data_start_code = 0x000001%02X\n", getbits(8));
  while (nextbits(24) != START_CODE_PREFIX)
  {
    getbits(8);
    user_data_bytes++;
  }
  next_start_code();
}

void extension_and_user_data(int i)
{
  while ((nextbits(8) == EXTENSION_START_CODE) ||
         (nextbits(8) == USER_DATA_START_CODE))
  {
    if (i != 1)
      if (nextbits(8) == EXTENSION_START_CODE)
        extension_data(i);
    if (nextbits(8) == USER_DATA_START_CODE)
      user_data();
  }
}

void group_of_pictures_header()
{
  zprintf(2, "\ngroup_start_code = 0x000001%02X\n", getbits(8));
  //zprintf(1, "  GOPS SO FAR = %d\n", GOPSSoFar);
  zprintf(1, "  Current GOP = %d", GOPSSoFar);
  GOPSSoFar++;
  completedFrames = framesSoFar;
  nextbits(25);
  if (timecode==1) {
	setTC(new_timecode(completedFrames,framerate, drop_frame));
  }
  else { getbits(25); }
  //zprintf(1, "  COMPLETED FRAMES SO FAR = %d\n", completedFrames);
  zprintf(1, "  Current Frame = %d\r", completedFrames);
  //zprintf(2, "  drop_frame_flag = %d\n", get1bit());
  //zprintf(2, "  time_code_hours = %d\n", getbits(5));
  //zprintf(2, "  time_code_minutes = %d\n", getbits(6));
  //zprintf(2, "  marker_bit = %d\n", get1bit());
  //zprintf(2, "  time_code_seconds = %d\n", getbits(6));
  //zprintf(2, "  time_code_pictures = %d\n", getbits(6));
  //zprintf(2, "  PROJECTED TC = %d\n", new_timecode(completedFrames));
  zprintf(2, "  closed_gop = %d\n", get1bit());
  zprintf(2, "  broken_link = %d\n", get1bit());
  next_start_code();
}

void picture_header()
{
  int i;

  zprintf(2, "\npicture_header = 0x000001%02X\n", getbits(8));
  temporalDisplay = getbits(10);
  zprintf(2, "  temporal_reference = %d\n", temporalDisplay);
  i = getbits(3);
  zprintf(2, "  picture_coding_type = %d\n", i);
  zprintf(2, "  vbv_delay = %d\n", getbits(16));
  if (i == 2 || i == 3)
  {
    zprintf(2, "  full_pel_forward_vector = %d\n", get1bit());
    zprintf(2, "  forward_f_code = %d\n", getbits(3));
  }
  if (i == 3)
  {
    zprintf(2, "  full_pel_backward_vector = %d\n", get1bit());
    zprintf(2, "  backward_f_code = %d\n", getbits(3));
  }
  while (nextbits(1) == 1)
  {
    zprintf(2, "  extra_bit_picture = %d\n", get1bit());
    zprintf(2, "  extra_information_picture = %d\n", getbits(8));
  }
  zprintf(2, "  extra_bit_picture = %d\n", get1bit());
  next_start_code();
}

void picture_coding_extension()
{
  int i;

  zprintf(2, "\nextension_start_code = 0x000001%02X\n", getbits(8));
  zprintf(2, "  picture_coding_extension_id = %01X\n", getbits(4));
  zprintf(2, "  f_code[0][0] forward horizontal = %d\n", getbits(4));
  zprintf(2, "  f_code[0][1] forward vertical = %d\n", getbits(4));
  zprintf(2, "  f_code[1][0] backward horizontal = %d\n", getbits(4));
  zprintf(2, "  f_code[1][1] backward vertical = %d\n", getbits(4));
  zprintf(2, "  intra_dc_precision = %d\n", getbits(2));
  picture_structure = getbits(2);
  zprintf(2, "  picture_structure = %d\n", picture_structure);
  zprintf(2, "  REAL CURRENT FRAME = %d\n", CurrentFrame(temporalDisplay,completedFrames));
  if ((telecine==true) || (tff!=NULL)) {
	zprintf(2, "INSERTING BITS: %d\n", put1bit(GetTFF(InsertPulldown(currentFrame))));
  }
  zprintf(2, "  top_field_first = %d", get1bit());
  //zprintf(2, " - WILL BE %d\n", GetTFF(InsertPulldown(currentFrame)));
  zprintf(2, "  frame_pred_frame_dct = %d\n", get1bit());
  zprintf(2, "  concealment_motion_vectors = %d\n", get1bit());
  zprintf(2, "  q_scale_type= %d\n", get1bit());
  zprintf(2, "  intra_vlc_format = %d\n", get1bit());
  zprintf(2, "  alternate_scan = %d\n", get1bit());
  if (telecine==true) {
	zprintf(2, "INSERTING BITS: %d\n", put1bit(GetRFF(InsertPulldown(currentFrame))));
  }
  if (norff==true) { put1bit(0); }
  repeat_first_field = get1bit();
  zprintf(2, "  FRAMES SO FAR = %d\n", framesSoFar);
  zprintf(2, "  repeat_first_field = %d", repeat_first_field);
  //zprintf(2, " - WILL BE %d\n", GetRFF(InsertPulldown(currentFrame)));
  if (prog_frames) { 
	  if (prog_frames==1) { put1bit(1); }
	  else { put1bit(0); }
  }
  else {
	if (telecine) {
	put1bit(GetRFF(InsertPulldown(currentFrame)));
	}
  }
  zprintf(2, "  chroma_420_type = %d\n", get1bit());
  if (prog_frames) { 
	  if (prog_frames==1) { put1bit(1); }
	  else { put1bit(0); }
  }
   else {
	if (telecine) {
	put1bit(GetRFF(InsertPulldown(currentFrame)));
	}
  }
  zprintf(2, "  progressive_frame = %d\n", get1bit());
  framesSoFar++;
  i = get1bit();
  zprintf(2, "  composite_display_flag = %d\n", i);
  if (i)
  {
    zprintf(2, "  v_axis = %d\n", get1bit());
    zprintf(2, "  field_sequence = %d\n", getbits(3));
    zprintf(2, "  sub_carrier = %d\n", get1bit());
    zprintf(2, "  burst_amplitude = %d\n", getbits(7));
    zprintf(2, "  sub_carrier_phase = %d\n", getbits(8));
  }
  next_start_code();
}

//void macroblock()
//{
//  while (nextbits(11) == 0x80)
//    zprintf(2, "  macroblock_escape = %d\n", getbits(11));
//
//}

void slice()
{
  slice_vertical_position = getbits(8);
  zprintf(3, "\nslice_start_code = 0x000001%02X\n", slice_vertical_position);
  if (vertical_size > 2800)
  {
    slice_vertical_position_extension = getbits(3);
    zprintf(3, "  slice_vertical_position_extension = %d\n", slice_vertical_position_extension);
	 mb_row = (slice_vertical_position_extension << 7) + slice_vertical_position - 1;
  }
  else
	 mb_row = slice_vertical_position - 1;
  previous_macroblock_address = (mb_row * mb_width) - 1;

  if (sequence_scalable_extension_present == 1)
    if (scalable_mode == DATA_PARTITIONING)
      zprintf(3, "  priority_breakpoint = %d\n", getbits(7));
  zprintf(3, "  quantiser_scale_code = %d\n", getbits(5));
  if (nextbits(1) == 1)
  {
    zprintf(3, "  intra_slice_flag = %d\n", get1bit());
    zprintf(3, "  intra_slice = %d\n", get1bit());
    zprintf(3, "  reserve_bits = %d\n", getbits(7));
    while (nextbits(1) == 1)
    {
      zprintf(3, "  extra_bit_slice = %d\n", get1bit());
      zprintf(3, "  extra_information_slice = %d\n", getbits(8));
    }
  }
  zprintf(3, "  extra_bit_slice = %d\n", get1bit());
//  do
//    macroblock();
//  while (nextbits(23) != 0);
  next_start_code();
}

void picture_data()
{
  do
    slice();
  while (nextbits(8) > 0 && nextbits(8) < 0xB0);
//  next_start_code();
}

int main(int argc, char* argv[])
{
  char tmpStr[256];
  //char* tmp2;

  col = 0;
  verbose_level = 1;
  sequence_headers = 0;
  sequence_extensions = 0;
  user_data_bytes = 0;
  sequence_scalable_extension_present = 0;
  zprintf (1, "PULLDOWN - v0.99d, by Hard Code.\n", 0) ;
  zprintf (1, "[Based on sources by Brent Beyeler (beyeler@home.com)]\n\n", 0);
  time_start= time(NULL);
  if (argc < 2)
  {  
	  zprintf (1, "PULLDOWN is a tool for modifying the encoding flags in an MPEG-2 video\n",0);
	  zprintf (1, "stream.  It is most useful for turning a 23.976fps progressive stream into\n",0);
	  zprintf (1, "an SVCD or DVD-compliant 29.97fps NTSC interlaced stream via 2:3 pulldown.\n\n",0);
	  zprintf (1, "Using pulldown with only the source file name and no additional arguments\n",0);
	  zprintf (1, "creates a new file called PULLDOWN.M2V in your current directory.\n\n",0);
	  zprintf (1, "usage: pulldown.exe <source> <target> [options]\n\n",0);
	  zprintf (1, "options:\n\n",0);
	  zprintf (1, "-nopulldown : pulldown will not insert the necessary flags for 2:3 pulldown\n",0);
	  zprintf (1, "-framerate [23.976, 25, 29.97] (defaults to 29.97) : changes framerate\n",0);
	  zprintf (1, "-norff : strips out all RFF flags\n",0);
	  zprintf (1, "-tff [odd, even] : changes the field order\n",0);
	  zprintf (1, "-notimecodes : does not recalculate timecodes for the new framerate\n",0);
	  zprintf (1, "-prog_seq [p, i] : marks the entire MPEG stream as progressive or interlaced\n",0);
	  zprintf (1, "-prog_frames [p, i] : marks each frame as progressive or interalced\n",0);
	  zprintf (1, "-drop_frame [true, false] : Set drop_frame_flag for NTSC streams\n",0);
	  zprintf (1, "-aspect_ratio [1:1, 4:3, 16:9, 2.21:1] : Change aspect ratio\n",0);
	 exit (1);
  }
  if (argv[2] != NULL) { NewFileName=argv[2]; }
  init_getbits(argv[1]);
  strcpy(tmpStr, argv[1]);
  strlwr(tmpStr);
  printf ("SOURCE = %s", argv[1]);
  printf (", TARGET = %s\n\n", NewFileName);
  for (int k=3;k<argc;k++)
  {
	  if (strcmp(argv[k], "-nopulldown") == 0) {
		  if (drop_frame != true) { timecode=2; }
		  telecine = false;
		  prog_seq=3;
		  zprintf (1, " - Disabling RFF/TFF pulldown\n", 0); 
	  }
	  if (strcmp(argv[k], "-framerate") == 0) {
		  framerate=0;
		  if (strcmp(argv[k+1], "29.97") == 0)
		  {
			framerate = 29.97;
		  }
		  if (strcmp(argv[k+1], "25") == 0)
		  {
			framerate = 25;
		  }
		  if (strcmp(argv[k+1], "23.976") == 0)
		  {
			framerate = 23.976;
		  }
		  if (framerate == 0)
		  { 
			  printf (" - %s is not a valid framerate.", argv[k+1]);
			  return(1);
		  }
		  
		  printf (" - Setting framerate to: %s fps\n", argv[k+1]);
	  }
	  if (strcmp(argv[k], "-norff") == 0) { 
		  norff = true;
		  telecine = false;
		  zprintf (1, " - Stripping out RFF flags\n", 0); 
	  }
	  if (strcmp(argv[k], "-tff") == 0) { 
		  telecine = false;
		  if (strcmp(argv[k+1], "odd") == 0)
		  {
			  tff = 1;
			  zprintf (1, " - Setting field order to: ODD\n", 0);
		  }
		  if (strcmp(argv[k+1], "even") == 0)
		  {
			  tff = 2;
			  zprintf (1, " - Setting field order to: EVEN\n", 0);
		  }
	  }
	  if (strcmp(argv[k], "-notimecodes") == 0) { 
		  timecode = NULL;
		  zprintf (1, " - Not recalculating timecodes\n", 0); 
	  }
	  if (strcmp(argv[k], "-prog_seq") == 0) { 
		  if (strcmp(argv[k+1], "p") == 0)
		  {
			  prog_seq = 1;
			  zprintf (1, " - Setting all sequences to PROGRESSIVE\n", 0); 
		  }
		  if (strcmp(argv[k+1], "i") == 0)
		  {
			  prog_seq = 2;
			  zprintf (1, " - Setting all sequences to INTERLACED\n", 0); 
		  }
	  }
	  if (strcmp(argv[k], "-prog_frames") == 0) { 
		  if (strcmp(argv[k+1], "p") == 0)
		  {
		  prog_frames = 1;
		  zprintf (1, " - Setting all frames to PROGRESSIVE\n", 0); 
		  }
		  if (strcmp(argv[k+1], "i") == 0)
		  {
		  prog_frames = 2;
		  zprintf (1, " - Setting all frames to INTERLACED\n", 0); 
		  }
	  }
	  if (strcmp(argv[k], "-drop_frame") == 0) { 
		  if (strcmp(argv[k+1], "true") == 0)
		  {
		  drop_frame = true;
		  timecode=1;
		  zprintf (1, " - Enabling drop_frame_flag\n", 0); 
		  }
		  if (strcmp(argv[k+1], "false") == 0)
		  {
		  drop_frame = false;
		  timecode=1;
		  zprintf (1, " - Disabling drop_frame_flag\n", 0); 
		  }
	  }
	  if (strcmp(argv[k], "-aspect_ratio") == 0) { 
		  if (strcmp(argv[k+1], "4:3") == 0)
		  {
		  aspect_ratio = 2;
		  }
		  if (strcmp(argv[k+1], "1:1") == 0)
		  {
		  aspect_ratio = 1;
		  }
		  if (strcmp(argv[k+1], "16:9") == 0)
		  {
		  aspect_ratio = 3;
		  }
		  if (strcmp(argv[k+1], "2.21:1") == 0)
		  {
		  aspect_ratio = 4;
		  }
		  if (aspect_ratio>0) {
			printf (" - Setting aspect ratio to %s\n", argv[k+1]); 
		  }
	  }
  }	  
  if (telecine==true)
  {
	  zprintf (1, " - Enabling 2:3 pulldown\n", 0); 
	  if (argc < 4) { printf ("\n",0); }
  }
  if (argc > 3) { printf ("\n",0); }
  if ((norff==true) && (telecine==true))
  {
	  zprintf (1, "You can't use -norff in conjunction with -pulldown.\n",0);
	  return(1);
  }

  verbose_level = 2;

  next_start_code();
  if (nextbits(8) != SEQUENCE_HEADER_CODE)
  {
    printf("\nFile is not an MPEG Video Stream\n");
    exit(1);
  }

  sequence_header();

  if (nextbits(8) != EXTENSION_START_CODE)
    mpeg2 = 0;
  else
    mpeg2 = 1;

  if (mpeg2)
    sequence_extension();

  do
  {
    extension_and_user_data(0);
    do
    {
      if (nextbits(8) == GROUP_START_CODE)
      {
		//  if ((framesSoFar > 0) && (peak_bitrate < (bitcount()/(framesSoFar / framerate)))) {
	    //		  peak_bitrate = (bitcount()/(framesSoFar / framerate));
		//  }
		//gop_bytes = bitcount() - gop_bytes;
		//avg_bitrate=avg_bitrate + gop_bytes;
		zprintf (2, "**** GOP ****", 0);
        group_of_pictures_header();
        extension_and_user_data(1);
      }
      picture_header();
      if (mpeg2)
        picture_coding_extension();
      extension_and_user_data(2);
      picture_data();
    } 
	while ((nextbits(8) == PICTURE_START_CODE) ||
             (nextbits(8) == GROUP_START_CODE));
    if (nextbits(8) != SEQUENCE_END_CODE)
    {
      sequence_header();
      if (mpeg2)
        sequence_extension();
    }
  } while (nextbits(8) != SEQUENCE_END_CODE);
  zprintf(2, "\nsequence_end_code = 0x000001%02X\n", getbits(8));

  finish_getbits();
  
  zprintf(1, "                                                                          \r", 0);
  zprintf(1, "Total Frames = %d\n", framesSoFar);
  //zprintf(1, "\nTotal Sequences = %d", SequencesSoFar);
  zprintf(1, "\nTotal GOPS = %d", GOPSSoFar);
  if (telecine == true) {
	  avg_bitrate = bitcount()/(framesSoFar / 23.976); 
  }
  else {
	  avg_bitrate = bitcount()/(framesSoFar / framerate);
  }
  zprintf(1, "\n\nAverage bitrate = %d bits/sec", avg_bitrate);
  zprintf(1, " - (MPEG stamped @ %d bits/sec)", (max_bitrate * 400));
//  zprintf(1, "\nPeak bitrate = %d", peak_bitrate);
  zprintf(1, "\n", 0);
  time_end = time(NULL);
  zprintf(1,"\nTime taken = %d seconds...\n", (time_end-time_start));
  return (0);
}


